@@ -12,7 +12,7 @@ import copy
 
 import numpy as np
 
-from astromodels import u
+from astromodels import u, update_logging_level
 from astromodels.core.model import (CannotWriteModel, DuplicatedNode, Model,
                                     ModelFileExists)
 from astromodels.core.model_parser import *
@@ -24,6 +24,10 @@ from astromodels.sources.extended_source import ExtendedSource
 from astromodels.sources.particle_source import ParticleSource
 from astromodels.sources.point_source import PointSource
 
+update_logging_level("DEBUG")
+
+
+
 
 def _get_point_source(name="test"):
 
@@ -324,6 +328,15 @@ def test_links():
 
     # Now test the link
 
+    # This should print a warning, as trying to change the value of a linked parameters does not have any effect
+    
+    old_value = m.one.spectrum.main.Powerlaw.K
+    
+    
+    m.one.spectrum.main.Powerlaw.K = 1.23456
+
+    assert old_value == m.one.spectrum.main.Powerlaw.K
+    
 
     # This instead should work
     new_value = 1.23456
@@ -333,10 +346,9 @@ def test_links():
 
     # Now try to remove the link
 
-    # First we remove it from the wrong parameters, which should issue a warning
-    # with pytest.warns(RuntimeWarning):
+    #    First we remove it from the wrong parameters, which should issue a warning
 
-    #     m.unlink(m.two.spectrum.main.Powerlaw.K)
+    m.unlink(m.two.spectrum.main.Powerlaw.K)
 
     # Remove it from the right parameter
 
@@ -643,8 +655,10 @@ def test_input_output_with_complex_functions():
 
     # Now set up the synch. spectrum for our source and the source itself
 
-    synch_spectrum = _ComplexTestFunction()
+    synch_spectrum = _ComplexTestFunction(file_name="test.txt")
 
+    synch_spectrum.dummy = "love"
+    
     # Use the particle distribution we created as source for the electrons
     # producing synch. emission
 
@@ -654,6 +668,58 @@ def test_input_output_with_complex_functions():
         "synch_source", ra=12.6, dec=-13.5, spectral_shape=synch_spectrum
     )
 
+
+    
+    
+    my_model = Model(electrons, synch_source)
+
+    my_model.display()
+
+    my_model.save("__test.yml")
+
+    new_model = load_model("__test.yml")
+
+    assert len(new_model.sources) == len(my_model.sources)
+
+    assert (
+        my_particle_distribution.index.value
+        == new_model.electrons.spectrum.main.shape.index.value
+    )
+
+    assert new_model.synch_source.spectrum.main.shape.file_name.value == "test.txt"
+
+    assert new_model.synch_source.spectrum.main.shape.dummy.value == "love"
+    
+    
+    os.remove("__test.yml")
+
+def test_input_output_with_complex_functions_as_composites():
+
+    my_particle_distribution = Powerlaw()
+
+    my_particle_distribution.index = -1.52
+
+    electrons = ParticleSource(
+        "electrons", distribution_shape=my_particle_distribution)
+
+    # Now set up the synch. spectrum for our source and the source itself
+
+    synch_spectrum = _ComplexTestFunction(file_name="test.txt")
+    
+    
+    # Use the particle distribution we created as source for the electrons
+    # producing synch. emission
+
+    synch_spectrum.particle_distribution = my_particle_distribution
+
+    photon_spec = synch_spectrum + Powerlaw()
+
+    photon_spec.dummy_1 = "love"
+    
+    synch_source = PointSource(
+        "synch_source", ra=12.6, dec=-13.5, spectral_shape=photon_spec
+    )
+
     my_model = Model(electrons, synch_source)
 
     my_model.display()
@@ -670,6 +736,14 @@ def test_input_output_with_complex_functions():
     )
 
 
+    assert new_model.synch_source.spectrum.main.shape.file_name_1.value == "test.txt"
+
+    assert new_model.synch_source.spectrum.main.shape.dummy_1.value == "love"
+
+    
+    os.remove("__test.yml")
+    
+
 def test_add_remove_sources():
 
     mg = ModelGetter()
